<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use PhpOffice\PhpSpreadsheet\Reader\Csv;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use Symfony\Component\HttpFoundation\StreamedResponse;

use App\Service\AuthService;
use App\Entity\ClientPetEntity;
use App\Entity\ClientPetPhotoEntity;

use App\Form\ClientPetForm;
use App\Entity\BranchEntity;
use App\Entity\SpeciesEntity;
use App\Entity\BreedEntity;


/**
 * @Route("/client_pet")
 */
class ClientPetController extends AbstractController
{

      /**
    * @Route("", name="client_pet_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Client Pet'))) return $authService->redirectToHome();
      
      $page_title = ' Pet'; 
      return $this->render('ClientPet/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/client_pet/index.js') ]
      );
   }

     /**
    * @Route("/details/{id}", name="client_pet_details")
    */
    public function details(Request $request,$id , AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       if(!$authService->isUserHasAccesses(array('Client Details'))) return $authService->redirectToHome();
       
       $page_title = ' Pet Details'; 
       $em = $this->getDoctrine()->getManager();

       $clientPet = $em->getRepository(ClientPetEntity::class)->find(base64_decode($id));
       
       if(!$clientPet) {
          $clientPet = new ClientPetEntity();
       }

       $branch = $authService->getUser()->getBranch();
       $branchBreeds = $em->getRepository(BreedEntity::class)->findBy(['branch' => $branch, 'isDeleted' => false]);
       $branchSpecieses = $em->getRepository(SpeciesEntity::class)->findBy(['branch' => $branch, 'isDeleted' => false]);

       $formOptions = array('action' => 'u', 'branchId' => $branch->getIdEncoded(), 'breeds' => $branchBreeds, 'species' => $branchSpecieses );
       $form = $this->createForm(ClientPetForm::class, $clientPet, $formOptions);

       if($request->getMethod() === 'POST') {

         $clientPet_form = $request->get($form->getName());
         $result = $this->processForm($clientPet_form, $clientPet, $form, $request);

         if($result['success']) {
           
                 return $this->redirect($this->generateUrl('client_pet_details', array(
                     'id' => base64_encode($result['id'])
                 )), 302);
          
         } else {
             $form->submit($clientPet_form, false);
         }
     }


       return $this->render('ClientPet/detail.html.twig', [ 
          'page_title' => $page_title,
          'form' => $form->createView(),
          'clientPet' => $clientPet,
          'id' => $id,
          'javascripts' => array('/plugins/datatables/jquery.dataTables.js') 

       ]);
    }

    private function processForm($clientPet_form, $clientPet ,$form, Request $request) {

      $em = $this->getDoctrine()->getManager();

      $errors = $em->getRepository(ClientPetEntity::class)->validate($clientPet_form, $clientPet->getClient());

      if(!count($errors)) {

         $form->handleRequest($request);

         if ($form->isValid()) {

            
               
               $em->persist($clientPet);
               $em->flush();

               $this->processRemove($clientPet_form, $clientPet, $em);
               $this->processFile($clientPet, $em);

               $this->get('session')->getFlashBag()->set('success_messages', 'Record Successfully Updated.');

               $result['redirect'] = 'index';
               $result['id'] = $clientPet->getId();

         } else {

               $this->get('session')->getFlashBag()->set('error_messages', 'Something went wrong. Please try again.');
               $result['redirect'] = 'form with error';
         }

         

          $result['success'] = true;

      } else {

          foreach($errors as $error) {
              $this->get('session')->getFlashBag()->add('error_messages', $error);
          }

          $result['redirect'] = 'index';
          $result['success'] = false;
      }

      return $result;
  }

       /**
     * @Route("/ajax_detail", name="client_ajax_detail")
     */
    public function ajaxDetail(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
        $em = $this->getDoctrine()->getManager();
        $id = $request->query->get('id');
        $clientPet = $em->getRepository(ClientPetEntity::class)->find(base64_decode($id)); 
        

        if(!$clientPet){

            $result['success'] = false;
            $result['msg'] = 'Opps something went wrong please try again later';
        } else {

       
            $result['html'] = $this->renderView('ClientPet/ajax_detail.html.twig', [
                  'page_title' => 'Pet Details',
                  'clientPet' => $clientPet

            ]);
        }
      
       return new JsonResponse($result);
    }

       /**
    * @Route("/ajax_list", name="client_pet_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(ClientPetEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

   
   private function processFile($clientPet, $em){
     
      if(isset($_FILES) && isset($_FILES['client_pet_form']) && isset($_FILES['client_pet_form']['name']) && isset($_FILES['client_pet_form']['name']['photo']) && count($_FILES['client_pet_form']['name']['photo'])){
         
         $photos = $_FILES['client_pet_form']['name']['photo'];
         foreach($photos  as $k => $photo){
            if(!empty($photo)){

               $clientPetPhoto = new ClientPetPhotoEntity();
               $baseName = $clientPet->getId() . '-' . time() . $k . '.' . pathinfo($_FILES['client_pet_form']['name']['photo'][$k], PATHINFO_EXTENSION);
               $uploadFile = $clientPet->getUploadRootDir() . '/' . $baseName;

               if(move_uploaded_file($_FILES['client_pet_form']['tmp_name']['photo'][$k], $uploadFile)) {
                  $clientPetPhoto->setFileDescription($_FILES['client_pet_form']['name']['photo'][$k]);
                  $clientPetPhoto->setParsedFileDescription($baseName);
                  $clientPetPhoto->setClientPet($clientPet);
                  $em->persist($clientPetPhoto);
               }

               $em->flush();
            }
         }
      
    
      }  

      
   }

   
   private function processRemove($form, $clietPet, $em){
 
      $photoIds = isset($form['photos']) ? $form['photos'] : []; 

      foreach($clietPet->getClientPetPhotos() as $photo){
         if(!in_array($photo->getIdEncoded(), $photoIds)){
            $photo->setIsDeleted(true);
            $em->flush();
         }
      } 
   }

   
    /**
    * @Route("/autocomplete_pet", name="client_pet_autocomplete")
    */
    public function petAutocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'clientPets',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(ClientPetEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }


}
