<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Knp\Snappy\Pdf;
use Symfony\Component\HttpFoundation\Response;

use App\Service\AuthService;
use App\Entity\GroomingEntity;
use App\Entity\GroomingPhotoEntity;
use App\Entity\GroomingServiceEntity;
use App\Entity\ServiceEntity;
use App\Entity\UserEntity;
use App\Entity\RoomEntity;
use App\Entity\SmsEntity;
use App\Entity\BranchSmsEntity;
use App\Entity\UseServiceItemEntity;
use App\Entity\InventoryEntity;
use App\Entity\CancelledServiceItemEntity;

use App\Form\GroomingForm;


/**
 * @Route("/grooming")
 */
class GroomingController extends AbstractController
{

   /**
     * @Route("/ajax_sales_info", name="grooming_ajax_sales_info")
     */
    public function ajaxSalesInfo(Request $request, AuthService $authService): JsonResponse
    {
       
        $result = [ 'success' =>  true, 'msg' => ''];

         if(!$authService->isLoggedIn()) { 
            $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';
         } else {
            $result['salesInfo'] = $this->getDoctrine()->getManager()->getRepository(GroomingEntity::class)->getSalesInfo($this->get('session')->get('userData'));

         }
         return new JsonResponse($result);
     }

   /**
    * @Route("/ajax_sales_analytics", name="grooming_ajax_sales_analytics")
    */
   public function ajaxSalesAnalytics(Request $request, AuthService $authService): JsonResponse
   {
      
      $result = [ 'success' =>  true, 'msg' => ''];

      if(!$authService->isLoggedIn()) { 
         $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';
      } else {
         $result['salesInAnalytics'] = $this->getDoctrine()->getManager()->getRepository(GroomingEntity::class)->getSalesAnalytics($request->query->all(), $this->get('session')->get('userData'));

      }

      $days = [];
      $stats = [];

      foreach($result['salesInAnalytics'] as $stat){
       
          array_push($days, $stat['dayMonth']);
          array_push($stats, round($stat['amount']));
      }

      $result['days'] = $days;
      $result['stats'] = $stats;
      $result['success'] = true;

      return new JsonResponse($result);
   }
    
   

   
   /**
    * @Route("", name="grooming_index")
    */
    public function index(Request $request, AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       if(!$authService->isUserHasAccesses(array('Grooming'))) return $authService->redirectToHome();
       
       $page_title = ' Grooming'; 
       return $this->render('Grooming/index.html.twig', [ 
          'page_title' => $page_title, 
          'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/grooming/index.js') ]
       );
   }
    
   /**
    * @Route("/dashboard", name="grooming_dashboard")
    */
    public function dashboard(Request $request, AuthService $authService)
    {
       
       if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
       if(!$authService->isUserHasAccesses(array('Grooming Dashboard'))) return $authService->redirectToHome();
       
       $page_title = ' Grooming Dashboard'; 
       return $this->render('Grooming/dashboard.html.twig', [ 
          'page_title' => $page_title, 
          'javascripts' => array('/js/grooming/dashboard.js') ]
       );
    }

   /**
    * @Route("/ajax_list", name="grooming_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(GroomingEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }


    /**
     * @Route("/ajax_form", name="grooming_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();

         $grooming = $em->getRepository(GroomingEntity::class)->find(base64_decode($formData['id']));
         
         if(!$grooming) {
            $grooming = new GroomingEntity();
         }
  
         $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
         $form = $this->createForm(GroomingForm::class, $grooming, $formOptions);
      
         $formChoices = [
            'serviceStatuses' => $authService->serviceStatuses()
         ];
         $result['html'] = $this->renderView('Grooming/ajax_form.html.twig', [
              'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Grooming',
              'form' => $form->createView(),
              'action' => $formData['action'],
              'javascripts' => array('/js/grooming/ajax_form.js'),
              'grooming' => $grooming,
              'formChoicesInarray' => $formChoices,
              'formChoices' => json_encode($formChoices),

            ]);
       
     

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="grooming_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       $user = $authService->getUser();
       if($request->getMethod() == "POST"){
         $groomingForm = $request->request->get('grooming_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(GroomingEntity::class)->validate($groomingForm);

         if(!count($errors)){
            
          
            $grooming = $em->getRepository(GroomingEntity::class)->find($groomingForm['id']);
            
            if(!$grooming) {
               $grooming = new GroomingEntity();
            }
   
            $formOptions = array('action' => $groomingForm['action'], 'branchId' => $user->getBranch()->getIdEncoded());
            $form = $this->createForm(GroomingForm::class, $grooming, $formOptions);
         
            switch($groomingForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $grooming->setStatus('Active');
                        $grooming->setTransactionNo($this->generateTransactionNo($user, $em));
                        $em->persist($grooming);
                        $em->flush();

                        $this->processServices($groomingForm, $grooming, $em, $user);

                        $em->refresh($grooming);
                        $grooming->setBilledAmount($grooming->getTotalServiceCharge());
                        $em->flush();

                        $this->processFile($grooming, $em);
   
                        $result['msg'] = 'Record successfully added to record.';
                        $result['id'] = $grooming->getIdEncoded();
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);


                     if ($form->isValid()) {

                        $em->persist($grooming);
                        $em->flush();
                        $this->processRemove($groomingForm, $grooming, $em);
                        
                        $isServiceCompleted = $this->processServices($groomingForm, $grooming, $em, $user);

                        $em->refresh($grooming);
                        $grooming->setBilledAmount($grooming->getTotalServiceCharge());
                        $em->flush();

                        if($grooming->getBilledAmount() > $grooming->getTotalPayment() ||  !$isServiceCompleted){

                           $grooming->setStatus('Active');
                        } else {
                           $grooming->setStatus('Completed');
                        }

                        $em->flush();
                        $this->processFile($grooming, $em);

                        $result['msg'] = 'Record successfully updated.';
                        $result['id'] = $grooming->getIdEncoded();
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $grooming->setIsDeleted(true);
                        $em->flush();
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
   }

   private function processServices($form, $grooming, $em, $user){

      $isComplete  = true;

      if(isset($form['groomingService'])){
         
         foreach ($form['groomingService'] as $key => $i) {
            if(isset($i['grooming_service_id'])){
               $groomingService = $em->getRepository(GroomingServiceEntity::class)->find(base64_decode($i['grooming_service_id']));
            } else {

               $groomingService =  new GroomingServiceEntity();
               
            }   

            $groomingService->setService($em->getReference(ServiceEntity::class, $i['service']));
            $groomingService->setUser($em->getReference(UserEntity::class, $i['staff']));
            $groomingService->setStatus($i['status']);
            $groomingService->setServiceRate($i['price']);
            $groomingService->setRemarks($i['remarks']);

            $groomingService->setGrooming($grooming);
            $em->persist($groomingService);
            $em->flush();

            $this->processServiceItemInventory($groomingService, $em);


            if($groomingService->getStatus() == 'In Progress'){
               $smsOnProcesses = $em->getRepository(SmsEntity::class)->findBy([
                  'isDeleted' => false,
                  'company' => $user->getBranch()->getCompany(),
                  'isServiceOnProcess' => true, 
               ]);
   
               foreach($smsOnProcesses as $sms){
                  
                  $branchSms = $em->getRepository(BranchSmsEntity::class)->findOneBy(['groomingService' => $groomingService, 'sms' => $sms]);
                  if(!$branchSms){

                     $branchSms =  new BranchSmsEntity();
                     $branchSms->setSms($sms);
                     $branchSms->setGroomingService($groomingService);
                     $branchSms->setMessage(str_replace('[status]', $groomingService->getStatus(), str_replace('[service]', $groomingService->getService()->getDescription(), str_replace('[pet_name]', $groomingService->getGrooming()->getClientPet()->getPetName(),str_replace('[client]', $groomingService->getGrooming()->getClientPet()->getClient()->getFullName(), $sms->getMessage())))));
                     $branchSms->setStatus('New');
                     $branchSms->setContactNo($groomingService->getGrooming()->getClientPet()->getClient()->getContactNo());
                     $branchSms->setBranch($user->getBranch());
                     $em->persist($branchSms);
                     $em->flush();
                  }
               }
            }

            if($groomingService->getStatus() == 'Completed'){
              
               $smsOnCompleteds = $em->getRepository(SmsEntity::class)->findBy([
                  'isDeleted' => false,
                  'company' => $user->getBranch()->getCompany(),
                  'isServiceCompleted' => true, 
                  
               ]);
   
               foreach($smsOnCompleteds as $sms){
                  $branchSms = $em->getRepository(BranchSmsEntity::class)->findOneBy(['groomingService' => $groomingService, 'sms' => $sms]);
                  if(!$branchSms){

                     $branchSms =  new BranchSmsEntity();
                     $branchSms->setSms($sms);
                     $branchSms->setGroomingService($groomingService);
                     $branchSms->setMessage(str_replace('[status]', $groomingService->getStatus(), str_replace('[service]', $groomingService->getService()->getDescription(), str_replace('[pet_name]', $groomingService->getGrooming()->getClientPet()->getPetName(),str_replace('[client]', $groomingService->getGrooming()->getClientPet()->getClient()->getFullName(), $sms->getMessage())))));
                     $branchSms->setStatus('New');
                     $branchSms->setContactNo($groomingService->getGrooming()->getClientPet()->getClient()->getContactNo());
                     $branchSms->setBranch($user->getBranch());
                     $em->persist($branchSms);
                     $em->flush();
                  }
               }
            } 

            if($i['status'] != 'Completed'){
               $isComplete = false;
            }

         }
         
      }

      return $isComplete;
   }

    /**
    * @Route("/trasaction_ajax_list", name="grooming_trasaction_ajax_list")
    */
    public function trasaction_ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(GroomingEntity::class)->transaction_ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

   private function processRemove($form, $grooming, $em){

      $serviceIds = isset($form['grooming_service_ids']) ? $form['grooming_service_ids'] : []; 
      $photoIds = isset($form['photos']) ? $form['photos'] : []; 
      //delete client organization
      foreach($grooming->getGroomingServices() as $groomingService){
         if(!in_array($groomingService->getIdEncoded(), $serviceIds)){
            $groomingService->setIsDeleted(true);
            $em->flush();

            $em->refresh($groomingService);
            $this->processCancelledServiceItem($groomingService, $em);
         }
      } 

      foreach($grooming->getGroomingPhotos() as $photo){
         if(!in_array($photo->getIdEncoded(), $photoIds)){
            $photo->setIsDeleted(true);
            $em->flush();
         }
      } 
   }

   private function processFile($grooming, $em){
     
      if(isset($_FILES) && isset($_FILES['grooming_form']) && isset($_FILES['grooming_form']['name']) && isset($_FILES['grooming_form']['name']['photo']) && count($_FILES['grooming_form']['name']['photo'])){
         
         $photos = $_FILES['grooming_form']['name']['photo'];
         foreach($photos  as $k => $photo){
            if(!empty($photo)){

               $groomingPhoto = new GroomingPhotoEntity();
               $baseName = $grooming->getId() . '-' . time() . $k . '.' . pathinfo($_FILES['grooming_form']['name']['photo'][$k], PATHINFO_EXTENSION);
               $uploadFile = $grooming->getUploadRootDir() . '/' . $baseName;

               if(move_uploaded_file($_FILES['grooming_form']['tmp_name']['photo'][$k], $uploadFile)) {
                  $groomingPhoto->setFileDescription($_FILES['grooming_form']['name']['photo'][$k]);
                  $groomingPhoto->setParsedFileDescription($baseName);
                  $groomingPhoto->setGrooming($grooming);
                  $em->persist($groomingPhoto);
               }

               $em->flush();
            }
         }
      
    
      }  

      
   }

   private function processCancelledServiceItem($groomingService, $em){
      $service = $groomingService->getService();

      $serviceItems = $service->getServiceItems();
      if(count($serviceItems)){

         foreach ($serviceItems as $serviceItem) {
            if(!$serviceItem->getisDeleted()){

               if($groomingService->getStatus() == 'Completed'){
                 
                  $cancelledServiceItem = $em->getRepository(CancelledServiceItemEntity::class)->findOneBy(['serviceItem' => $serviceItem, 'groomingService' => $groomingService]);
                  $inventory = $em->getRepository(InventoryEntity::class)->findOneBy(['branch' => $groomingService->getGrooming()->getBranch(), 'item' => $serviceItem->getItem()]);

                  if(!$cancelledServiceItem){
                     $cancelledServiceItem = new CancelledServiceItemEntity();
                  }
   
                  $cancelledServiceItem->setServiceItem($serviceItem);
                  $cancelledServiceItem->setGroomingService($groomingService);
                  $cancelledServiceItem->setQuantity($serviceItem->getQuantity());
                  $cancelledServiceItem->setInventory($inventory);

                  $em->persist($cancelledServiceItem);
                  $em->flush();

                  $inventory->setEndingQuantity($inventory->getEndingQuantity() + $cancelledServiceItem->getQuantity());
                  $em->flush();
               }
            }
         }
      }
   }
   private function processServiceItemInventory($groomingService, $em){

      $service = $groomingService->getService();

      $serviceItems = $service->getServiceItems();
      if(count($serviceItems)){

         foreach ($serviceItems as $serviceItem) {
            if(!$serviceItem->getisDeleted()){

               if($groomingService->getStatus() == 'Completed'){
                 
                  $useServiceItem = $em->getRepository(UseServiceItemEntity::class)->findOneBy(['serviceItem' => $serviceItem, 'groomingService' => $groomingService ]);
                  $inventory = $em->getRepository(InventoryEntity::class)->findOneBy(['branch' => $groomingService->getGrooming()->getBranch(), 'item' => $serviceItem->getItem()]);
                  
                  if(!$useServiceItem){
                     $useServiceItem = new UseServiceItemEntity();
                  }
   
                  $useServiceItem->setServiceItem($serviceItem);
                  $useServiceItem->setGroomingService($groomingService);
                  $useServiceItem->setQuantity($serviceItem->getQuantity());
                  $useServiceItem->setInventory($inventory);

                  $em->persist($useServiceItem);
                  $em->flush();

                  $inventory->setEndingQuantity($inventory->getEndingQuantity() - $useServiceItem->getQuantity());
                  $em->flush();
               }
            }
         }
      }
   }
       /**
     * @Route("/invoice/pdf/{id}", name = "grooming_invoice_pdf")
     */
    public function invoicePdf(Request $request, AuthService $authService, Pdf $pdf, $id){

      ini_set('memory_limit', '2048M');

      $grooming  = $this->getDoctrine()->getManager()->getRepository(GroomingEntity::class)->find(base64_decode($id));
      $options = [
          'orientation' => 'Portrait',
          'print-media-type' =>  True
      ];


      $newContent = $this->renderView('Grooming/invoice.wkpdf.twig', array('grooming' => $grooming));

      $xml = $pdf->getOutputFromHtml($newContent,$options);

      $pdfResponse = array(
          'success' => true,
          'msg' => 'PDF was successfully generated.', 
          'pdfBase64' => base64_encode($xml)
      );
     
      $pdfContent = $pdfResponse['pdfBase64'];
       return new Response(base64_decode($pdfContent), 200, array(
          'Content-Type' => 'application/pdf',
          'Content-Disposition'   => 'attachment; filename="'.  'billing-invoice' .'-' . date('m/d/Y') . '.pdf"'
      ));
  }

   private function generateTransactionNo($user, $em){
      $lodges = $em->getRepository(GroomingEntity::Class)->todayGrooming($user);
      return 'G' . date('Ymd') .  (count( $lodges) + 1) ;
   }

}
