<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use App\Service\AuthService;
use App\Entity\InventoryAdjustmentEntity;

use App\Form\InventoryAdjustmentForm;



/**
 * @Route("/inventory_adjustment")
 */
class InventoryAdjustmentController extends AbstractController
{



    /**
     * @Route("/ajax_form", name="inventory_adjustment_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $inventoryAdjustment = $em->getRepository(InventoryAdjustmentEntity::class)->find(base64_decode($formData['id']));
       
       if(!$inventoryAdjustment) {
          $inventoryAdjustment = new InventoryAdjustmentEntity();
       }

       $formOptions = array('action' => $formData['action'], 'inventoryId' => $formData['inventoryId']);
       
       $form = $this->createForm(InventoryAdjustmentForm::class, $inventoryAdjustment, $formOptions);
    
       $result['html'] = $this->renderView('InventoryAdjustment/ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Inventory',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/inventory_adjustment/ajax_form.js') 
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="inventory_adjustment_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $inventoryAdjustmentForm = $request->request->get('inventory_adjustment_form');
         
         $em = $this->getDoctrine()->getManager();
        // $errors = $em->getRepository(InventoryAdjustmentEntity::class)->validate($inventoryAdjustmentForm);
         $errors =[];


         if(!count($errors)){
            
            $inventoryAdjustment = $em->getRepository(InventoryAdjustmentEntity::class)->find($inventoryAdjustmentForm['id']);
            
            if(!$inventoryAdjustment) {
               $inventoryAdjustment = new InventoryAdjustmentEntity();
            }
     
            $formOptions = array('action' => $inventoryAdjustmentForm['action'], 'inventoryId' => $inventoryAdjustmentForm['inventory']);
            $form = $this->createForm(InventoryAdjustmentForm::class, $inventoryAdjustment, $formOptions);


            switch($inventoryAdjustmentForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($inventoryAdjustment);
                        $em->flush();

                        $inventory = $inventoryAdjustment->getInventory();
                        if($inventoryAdjustment->getSellingPrice()){
                           $inventory->setSellingPrice($inventoryAdjustment->getSellingPrice());
                        }

                        if($inventoryAdjustment->getBuyingPrice()){
                           $inventory->setBuyingPrice($inventoryAdjustment->getBuyingPrice());
                        }

                        if($inventoryAdjustment->getAdjustmentType() == 'Add'){
                           $inventory->setEndingQuantity($inventory->getEndingQuantity() + $inventoryAdjustment->getQuantity());
                        }

                        if($inventoryAdjustment->getAdjustmentType() == 'Deduct'){
                           $inventory->setEndingQuantity($inventory->getEndingQuantity() - $inventoryAdjustment->getQuantity());
                        }

                        $em->flush();
   
                        $result['msg'] = 'Record successfully added to record.';
                        $result['id'] = $inventoryAdjustment->getIdEncoded();
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

   /**
    * @Route("/ajax_list", name="inventory_adjustment_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(InventoryAdjustmentEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

}
