<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use App\Service\AuthService;
use App\Entity\InventoryEntity;

use App\Form\InventoryForm;



/**
 * @Route("/inventory")
 */
class InventoryController extends AbstractController
{

   /**
     * @Route("/details/{id}", name="inventory_item_details")
     */
    public function details($id, Request $request, AuthService $authService)
    {
        if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
        if(!$authService->isUserHasAccesses(array('Inventory Details'))) return $authService->redirectToHome();
        
        $inventory = $this->getDoctrine()->getManager()->getRepository(InventoryEntity::class)->find(base64_decode($id));
        $page_title = ' Inventory Details'; 
       return $this->render('Inventory/details.html.twig', [ 
            'inventory' => $inventory,
            'page_title' => $page_title, 
            'javascripts' => array('/plugins/datatables/jquery.dataTables.js','/js/inventory/details.js') ]
       );
    }

    /**
     * @Route("/ajax_form", name="inventory_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $inventory = $em->getRepository(InventoryEntity::class)->find(base64_decode($formData['id']));
       
       if(!$inventory) {
          $inventory = new InventoryEntity();
       }

       $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
       
       $form = $this->createForm(InventoryForm::class, $inventory, $formOptions);
    
       $result['html'] = $this->renderView('Inventory/ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Inventory',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/inventory/ajax_form.js') 
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="inventory_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $inventoryForm = $request->request->get('inventory_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(InventoryEntity::class)->validate($inventoryForm);

         if(!count($errors)){
            
            $inventory = $em->getRepository(InventoryEntity::class)->find($inventoryForm['id']);
            
            if(!$inventory) {
               $inventory = new InventoryEntity();
            }
     
            $formOptions = array('action' => $inventoryForm['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
            $form = $this->createForm(InventoryForm::class, $inventory, $formOptions);


            switch($inventoryForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        $inventory->setEndingQuantity($inventoryForm['beginning_quantity']);
                        $em->persist($inventory);
                        $em->flush();
   
                        $result['msg'] = 'Record successfully added to record.';
                        $result['id'] = $inventory->getIdEncoded();
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);
                     if ($form->isValid()) {
      
                        $em->persist($inventory);
                        $em->flush();
                        $result['msg'] = 'Record successfully updated.';
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $inventory->setIsDeleted(true);
                        $em->flush();
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

      /**
    * @Route("", name="inventory_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Inventory'))) return $authService->redirectToHome();
      
      $page_title = ' Inventory'; 
      return $this->render('Inventory/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/inventory/index.js') ]
      );
   }



   /**
    * @Route("/autocomplete", name="inventory_autocomplete")
    */
   public function autocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'inventorys',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(InventoryEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }


   /**
    * @Route("/ajax_list", name="inventory_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(InventoryEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

}
