<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use App\Service\AuthService;
use App\Entity\ServiceEntity;
use App\Entity\ItemEntity;
use App\Entity\ServiceItemEntity;

use App\Form\ServiceForm;

/**
 * @Route("/service")
 */
class ServiceController extends AbstractController
{
    /**
     * @Route("/ajax_form", name="service_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $service = $em->getRepository(ServiceEntity::class)->find(base64_decode($formData['id']));
       
       if(!$service) {
          $service = new ServiceEntity();
       }

       $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
       $form = $this->createForm(ServiceForm::class, $service, $formOptions);
    
       $result['html'] = $this->renderView('Service/ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Service',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/service/ajax_form.js'), 
            'service' => $service
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="service_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $serviceForm = $request->request->get('service_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(ServiceEntity::class)->validate($serviceForm);

         if(!count($errors)){
            
            $service = $em->getRepository(ServiceEntity::class)->find($serviceForm['id']);
            
            if(!$service) {
               $service = new ServiceEntity();
            }
     
            $formOptions = array('action' => $serviceForm['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
            $form = $this->createForm(ServiceForm::class, $service, $formOptions);


            switch($serviceForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($service);
                        $em->flush();

                       $this->processItems($serviceForm, $service, $em);
   
                        $result['msg'] = 'Record successfully added to record.';
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);
                     if ($form->isValid()) {
      
                        $em->persist($service);
                        $em->flush();

                        $this->processRemove($serviceForm, $service, $em);
                        $this->processItems($serviceForm, $service, $em);
                        $result['msg'] = 'Record successfully updated.';
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $service->setIsDeleted(true);
                        $em->flush();
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

    private function processItems($form, $service, $em){
      if(isset($form['serviceItem'])){
         
         foreach ($form['serviceItem'] as $key => $i) {
            if(isset($i['service_item_id'])){
               $serviceItem = $em->getRepository(ServiceItemEntity::class)->find(base64_decode($i['service_item_id']));

               $serviceItem->setItem($em->getReference(ItemEntity::class, $i['item']));
               $serviceItem->setQuantity($i['quantity']);
               $serviceItem->setService($service);
               $em->persist($serviceItem);
               $em->flush();
            } else {
               

               $serviceItemEx = $em->getRepository(ServiceItemEntity::class)->findOneBy([
                  'service' => $service,
                  'item' => $em->getReference(ItemEntity::class, $i['item']),
                  'isDeleted' => false
               ]);

               if(!$serviceItemEx){

                  $serviceItem =  new ServiceItemEntity();
                  $serviceItem->setItem($em->getReference(ItemEntity::class, $i['item']));
                  $serviceItem->setQuantity($i['quantity']);
                  $serviceItem->setService($service);
                  $em->persist($serviceItem);
                  $em->flush();
               }
            }   
         }
         
      }
   }

   private function processRemove($form, $service, $em){

      $itemsIds = isset($form['service_item_ids']) ? $form['service_item_ids'] : []; 

      //delete client organization
      foreach($service->getServiceItems() as $item){
         if(!in_array($item->getIdEncoded(), $itemsIds)){
            $item->setIsDeleted(true);
            $em->flush();
         }
      } 
   }

      /**
    * @Route("", name="service_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Service'))) return $authService->redirectToHome();
      
      $page_title = ' Service'; 
      return $this->render('Service/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/service/index.js') ]
      );
   }



   /**
    * @Route("/autocomplete", name="service_autocomplete")
    */
   public function autocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'services',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(ServiceEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }

   /**
    * @Route("/grooming_autocomplete", name="service_grooming_autocomplete")
    */
    public function groomingAutocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'services',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(ServiceEntity::class)->grromingAutocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }

   /**
    * @Route("/ajax_list", name="service_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(ServiceEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

}
