<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use App\Entity\SmsEntity;
use App\Form\SmsForm;

use App\Service\AuthService;

/**
 * @Route("/admin/sms")
 */
class SmsController extends AbstractController
{


    /**
     * @Route("/ajax_form", name="sms_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $sms = $em->getRepository(SmsEntity::class)->find(base64_decode($formData['id']));
       
       if(!$sms) {
          $sms = new SmsEntity();
       }

       $formOptions = array('action' => $formData['action'], 'companyId' => $formData['companyId']);
       $form = $this->createForm(SmsForm::class, $sms, $formOptions);
    
       $result['html'] = $this->renderView('Company/sms_ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Sms',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/sms/ajax_form.js') 
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="sms_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $smsForm = $request->request->get('sms_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(SmsEntity::class)->validate($smsForm);

         if(!count($errors)){
            
            $sms = $em->getRepository(SmsEntity::class)->find($smsForm['id']);
            
            if(!$sms) {
               $sms = new SmsEntity();
            }

            $formOptions = array('action' => $smsForm['action'], 'companyId' => $smsForm['company']);     
            $form = $this->createForm(SmsForm::class, $sms, $formOptions);


            switch($smsForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($sms);
                        $em->flush();
   
                        $result['msg'] = 'Record successfully added to record.';
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);
                     if ($form->isValid()) {
      
                        $em->persist($sms);
                        $em->flush();
                        $result['msg'] = 'Record successfully updated.';
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $sms->setIsDeleted(true);
                        $em->flush();
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_list", name="sms_ajax_list")
     */
    public function ajax_listAction(Request $request, AuthService $authService) {

        $get = $request->query->all();

        $result = array(
            "draw" => intval($get['draw']),
            "recordsTotal" => 0,
            "recordsFiltered" => 0,
            "data" => array()
        );

        if($authService->isLoggedIn()) {
            $result = $this->getDoctrine()->getManager()->getRepository(SmsEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
        }

        return new JsonResponse($result);
    }

}
