<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RedirectResponse;

use App\Service\AuthService;
use App\Entity\SupplierEntity;
use App\Form\SupplierForm;

/**
 * @Route("/supplier")
 */
class SupplierController extends AbstractController
{
    /**
     * @Route("/ajax_form", name="supplier_ajax_form")
     */
    public function ajaxForm(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];
       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}
       
       $em = $this->getDoctrine()->getManager();
       $formData = $request->request->all();
       $supplier = $em->getRepository(SupplierEntity::class)->find(base64_decode($formData['id']));
       
       if(!$supplier) {
          $supplier = new SupplierEntity();
       }

       $formOptions = array('action' => $formData['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
       $form = $this->createForm(SupplierForm::class, $supplier, $formOptions);
    
       $result['html'] = $this->renderView('Supplier/ajax_form.html.twig', [
            'page_title' => ($formData['action'] == 'n' ? 'New' : 'Update') . '  Supplier',
            'form' => $form->createView(),
            'action' => $formData['action'],
            'javascripts' => array('/js/supplier/ajax_form.js') 
        ]);

       return new JsonResponse($result);
    }

    /**
     * @Route("/ajax_form_process", name="supplier_ajax_form_process")
     */
    public function ajaxFormProcess(Request $request, AuthService $authService): JsonResponse
    {
       
       $result = [ 'success' =>  true, 'msg' => ''];

       if(!$authService->isLoggedIn()) { $result['success'] = false; $result['msg'] = 'Unauthorized access please call a system administrator.';}

       if($request->getMethod() == "POST"){
         $supplierForm = $request->request->get('supplier_form');
         
         $em = $this->getDoctrine()->getManager();
         $errors = $em->getRepository(SupplierEntity::class)->validate($supplierForm);

         if(!count($errors)){
            
            $supplier = $em->getRepository(SupplierEntity::class)->find($supplierForm['id']);
            
            if(!$supplier) {
               $supplier = new SupplierEntity();
            }
     
            $formOptions = array('action' => $supplierForm['action'], 'branchId' => $authService->getUser()->getBranch()->getIdEncoded());
            $form = $this->createForm(SupplierForm::class, $supplier, $formOptions);


            switch($supplierForm['action']) {
               case 'n': // new

                     $form->handleRequest($request);
      
                     if ($form->isValid()) {
                        
                        $em->persist($supplier);
                        $em->flush();
   
                        $result['msg'] = 'Record successfully added to record.';
                     } else {
      
                        $result['error'] = 'Ooops something went wrong please try again later.';
                     }
      
                     break;
      
               case 'u': // update

                     $form->handleRequest($request);
                     if ($form->isValid()) {
      
                        $em->persist($supplier);
                        $em->flush();
                        $result['msg'] = 'Record successfully updated.';
                     } else {
                       
                        $result['error'] = 'Ooops2 something went wrong please try again later.';
                     }
      
                     break;
      
               case 'd': // delete
                     $form->handleRequest($request);
                     if ($form->isValid()) {
                          
                        $supplier->setIsDeleted(true);
                        $em->flush();
      
                        $result['msg'] = 'Record successfully deleted.';
      
                     } else {
      
                        $result['error'] = 'Ooops 3something went wrong please try again later.';
                     }
      
                     break;
            }
        
         } else {

             $result['success'] = false;
             $result['msg'] = '';
             foreach ($errors as $error) {
                 
                 $result['msg'] .= $error;
             }
         }
     } else {

         $result['error'] = 'Ooops something went wrong please try again later.';
     }
    
       return new JsonResponse($result);
    }

      /**
    * @Route("", name="supplier_index")
    */
   public function index(Request $request, AuthService $authService)
   {
      
      if(!$authService->isLoggedIn()) return $authService->redirectToLogin();
      if(!$authService->isUserHasAccesses(array('Supplier'))) return $authService->redirectToHome();
      
      $page_title = ' Supplier'; 
      return $this->render('Supplier/index.html.twig', [ 
         'page_title' => $page_title, 
         'javascripts' => array('plugins/datatables/jquery.dataTables.js','/js/supplier/index.js') ]
      );
   }



   /**
    * @Route("/autocomplete", name="supplier_autocomplete")
    */
   public function autocompleteAction(Request $request) {

      return new JsonResponse(array(
         'query' => 'suppliers',
         'suggestions' => $this->getDoctrine()->getManager()->getRepository(SupplierEntity::class)->autocompleteSuggestions($request->query->all(), $this->get('session')->get('userData'))
      ));
   }


   /**
    * @Route("/ajax_list", name="supplier_ajax_list")
    */
    public function ajax_listAction(Request $request, AuthService $authService) {

      $get = $request->query->all();

      $result = array(
          "draw" => intval($get['draw']),
          "recordsTotal" => 0,
          "recordsFiltered" => 0,
          "data" => array()
      );

      if($authService->isLoggedIn()) {
          $result = $this->getDoctrine()->getManager()->getRepository(SupplierEntity::class)->ajax_list($get, $this->get('session')->get('userData'));
      }

      return new JsonResponse($result);
   }

}
