<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\ClientPetRepository")
 * @ORM\Table(name="client_pet")
 * @ORM\HasLifecycleCallbacks()
 */

class ClientPetEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="pet_name", type="string", nullable=true)
     */
    protected $petName;

    /**
     * @ORM\Column(name="remarks", type="string", nullable=true)
     */
    protected $remarks;

    /**
     * @ORM\Column(name="file_description", type="string", nullable=true)
     */
    protected $fileDescription;

    /**
     * @ORM\Column(name="parsed_file_description", type="string", nullable=true)
     */
    protected $parsedFileDescription;


    /**
     * @ORM\ManyToOne(targetEntity="SpeciesEntity", inversedBy="clientPets")
     * @ORM\JoinColumn(name="species_id", referencedColumnName="id", nullable=true)
     */
    protected $species;
    
    /**
     * @ORM\ManyToOne(targetEntity="BreedEntity", inversedBy="clientPets")
     * @ORM\JoinColumn(name="breed_id", referencedColumnName="id", nullable=true)
     */
    protected $breed;

    /**
     * @ORM\ManyToOne(targetEntity="ClientEntity", inversedBy="clientPets")
     * @ORM\JoinColumn(name="client_id", referencedColumnName="id", nullable=true)
     */
    protected $client;


    /**
     * @ORM\OneToMany(targetEntity="LodgeEntity", mappedBy="clientPet", cascade={"remove"})
     */
    protected $lodges;

    /**
     * @ORM\OneToMany(targetEntity="GroomingEntity", mappedBy="clientPet", cascade={"remove"})
     */
    protected $groomings;

    /**
     * @ORM\OneToMany(targetEntity="ScheduleEntity", mappedBy="clientPet", cascade={"remove"})
     */
    protected $schedules;

          /**
     * @ORM\OneToMany(targetEntity="ClientPetPhotoEntity", mappedBy="clientPet", cascade={"remove"})
     */
    protected $clientPetPhotos;

    public function __construct()
    {
        $this->lodges = new ArrayCollection();
        $this->groomings = new ArrayCollection();
        $this->schedules = new ArrayCollection();
        $this->clientPetPhotos = new ArrayCollection();
    }

    
    /*--------------------------------------------------------------------------------------------------------*/
    /*					Client Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

   


/**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return ClientEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /**
     * Remove the file from the disk
     *
     * @ORM\PreRemove
     */
    public function removeFile() {

        $file = $this->getUploadRootDir() . '/' . $this->parsedFileDescription;
        if(!empty($this->profilePic) && file_exists($file)) unlink($file);
    }

        /**
     * Get uploadDir
     *
     * @return string
     */
    public function getUploadDir() {

        return '/uploads/file';
    }

    /**
     * Get uploadRootDir
     *
     * @return string
     */
    public function getUploadRootDir() {

        return __DIR__ . './../../public' . $this->getUploadDir();
    }

        /**
     * get fileWebPath
     *
     * @return string
     */
    public function getFileWebPath() {

        $parsedDesc = $this->parsedFileDescription;
        $file = $this->getUploadRootDir() . '/' . $parsedDesc;
     
        if(!empty($parsedDesc) ) {
            return   $this->getUploadDir() . '/' . $parsedDesc;
        } else {
            return '';
        }

       
    }


    /*--------------------------------------------------------------------------------------------------------*/
    /*					    Defined Setters and Getters													      */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getPetName(): ?string
    {
        return $this->petName;
    }

    public function setPetName(?string $petName): self
    {
        $this->petName = $petName;

        return $this;
    }

    public function getSpecies(): ?SpeciesEntity
    {
        return $this->species;
    }

    public function setSpecies(?SpeciesEntity $species): self
    {
        $this->species = $species;

        return $this;
    }

    public function getBreed(): ?BreedEntity
    {
        return $this->breed;
    }

    public function setBreed(?BreedEntity $breed): self
    {
        $this->breed = $breed;

        return $this;
    }

    public function getClient(): ?ClientEntity
    {
        return $this->client;
    }

    public function setClient(?ClientEntity $client): self
    {
        $this->client = $client;

        return $this;
    }

    /**
     * @return Collection<int, LodgeEntity>
     */
    public function getLodges(): Collection
    {
        return $this->lodges;
    }

    public function addLodge(LodgeEntity $lodge): self
    {
        if (!$this->lodges->contains($lodge)) {
            $this->lodges[] = $lodge;
            $lodge->setClientPet($this);
        }

        return $this;
    }

    public function removeLodge(LodgeEntity $lodge): self
    {
        if ($this->lodges->removeElement($lodge)) {
            // set the owning side to null (unless already changed)
            if ($lodge->getClientPet() === $this) {
                $lodge->setClientPet(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, GroomingEntity>
     */
    public function getGroomings(): Collection
    {
        return $this->groomings;
    }

    public function addGrooming(GroomingEntity $grooming): self
    {
        if (!$this->groomings->contains($grooming)) {
            $this->groomings[] = $grooming;
            $grooming->setClientPet($this);
        }

        return $this;
    }

    public function removeGrooming(GroomingEntity $grooming): self
    {
        if ($this->groomings->removeElement($grooming)) {
            // set the owning side to null (unless already changed)
            if ($grooming->getClientPet() === $this) {
                $grooming->setClientPet(null);
            }
        }

        return $this;
    }

    public function getRemarks(): ?string
    {
        return $this->remarks;
    }

    public function setRemarks(?string $remarks): self
    {
        $this->remarks = $remarks;

        return $this;
    }

    public function getFileDescription(): ?string
    {
        return $this->fileDescription;
    }

    public function setFileDescription(?string $fileDescription): self
    {
        $this->fileDescription = $fileDescription;

        return $this;
    }

    public function getParsedFileDescription(): ?string
    {
        return $this->parsedFileDescription;
    }

    public function setParsedFileDescription(?string $parsedFileDescription): self
    {
        $this->parsedFileDescription = $parsedFileDescription;

        return $this;
    }

    /**
     * @return Collection<int, ScheduleEntity>
     */
    public function getSchedules(): Collection
    {
        return $this->schedules;
    }

    public function addSchedule(ScheduleEntity $schedule): self
    {
        if (!$this->schedules->contains($schedule)) {
            $this->schedules[] = $schedule;
            $schedule->setClientPet($this);
        }

        return $this;
    }

    public function removeSchedule(ScheduleEntity $schedule): self
    {
        if ($this->schedules->removeElement($schedule)) {
            // set the owning side to null (unless already changed)
            if ($schedule->getClientPet() === $this) {
                $schedule->setClientPet(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, ClientPetPhotoEntity>
     */
    public function getClientPetPhotos(): Collection
    {
        return $this->clientPetPhotos;
    }

    public function addClientPetPhoto(ClientPetPhotoEntity $clientPetPhoto): self
    {
        if (!$this->clientPetPhotos->contains($clientPetPhoto)) {
            $this->clientPetPhotos[] = $clientPetPhoto;
            $clientPetPhoto->setClientPet($this);
        }

        return $this;
    }

    public function removeClientPetPhoto(ClientPetPhotoEntity $clientPetPhoto): self
    {
        if ($this->clientPetPhotos->removeElement($clientPetPhoto)) {
            // set the owning side to null (unless already changed)
            if ($clientPetPhoto->getClientPet() === $this) {
                $clientPetPhoto->setClientPet(null);
            }
        }

        return $this;
    }

  
   
}
