<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\GroomingRepository")
 * @ORM\Table(name="grooming")
 * @ORM\HasLifecycleCallbacks()
 */

class GroomingEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="transaction_no", type="string")
     */
    protected $transactionNo;

    /**
     * @ORM\Column(name="status", type="string")
     */
    protected $status;

    /**
     * @ORM\Column(name="billed_amount", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $billedAmount;

    /**
     * @ORM\Column(name="is_ready_for_pick_up", type="boolean", nullable=true)
     */
    protected $isReadyForPickUp;

        /**
     * @ORM\Column(name="remarks", type="string", nullable=true)
     */
    protected $remarks;

    /**
     * @ORM\Column(name="file_description", type="string", nullable=true)
     */
    protected $fileDescription;

    /**
     * @ORM\Column(name="parsed_file_description", type="string", nullable=true)
     */
    protected $parsedFileDescription;

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="groomings")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\ManyToOne(targetEntity="ClientPetEntity", inversedBy="groomings")
     * @ORM\JoinColumn(name="client_pet_id", referencedColumnName="id", nullable=true)
     */
    protected $clientPet;

    /**
     * @ORM\OneToMany(targetEntity="GroomingServiceEntity", mappedBy="grooming", cascade={"remove"})
     */
    protected $groomingServices;

      /**
     * @ORM\OneToMany(targetEntity="GroomingPaymentEntity", mappedBy="grooming", cascade={"remove"})
     */
    protected $groomingPayments;

      /**
     * @ORM\OneToMany(targetEntity="GroomingPhotoEntity", mappedBy="grooming", cascade={"remove"})
     */
    protected $groomingPhotos;


    
    public function __construct($data = null)
    {
        $this->groomingServices = new ArrayCollection();
        $this->groomingPayments = new ArrayCollection();
        $this->groomingPhotos = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return GroomingEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /**
     * Get totalServiceCharge
     *
     *
     * @return decimal
     */
    public function getTotalServiceCharge()
    {
        $mt = 0;

        foreach($this->getGroomingServices() as $service){
            
            if(!$service->getIsDeleted()){
                $mt+= $service->getServiceRate();
            }
        }

        return $mt;
    }

     /**
     * Get totalPayment
     *
     *
     * @return decimal
     */
    public function getTotalPayment()
    {
        $mt = 0;

        foreach($this->getGroomingPayments() as $p){
            
            if(!$p->getIsDeleted()){
                $mt+= $p->getAmount();
            }
        }

        return $mt;
    }

        /**
     * Remove the file from the disk
     *
     * @ORM\PreRemove
     */
    public function removeFile() {

        $file = $this->getUploadRootDir() . '/' . $this->parsedFileDescription;
        if(!empty($this->profilePic) && file_exists($file)) unlink($file);
    }

        /**
     * Get uploadDir
     *
     * @return string
     */
    public function getUploadDir() {

        return '/uploads/file';
    }

    /**
     * Get uploadRootDir
     *
     * @return string
     */
    public function getUploadRootDir() {

        return __DIR__ . './../../public' . $this->getUploadDir();
    }

        /**
     * get fileWebPath
     *
     * @return string
     */
    public function getFileWebPath() {

        $parsedDesc = $this->parsedFileDescription;
        $file = $this->getUploadRootDir() . '/' . $parsedDesc;
     
        if(!empty($parsedDesc) ) {
            return   $this->getUploadDir() . '/' . $parsedDesc;
        } else {
            return '';
        }

       
    }


    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    public function getClientPet(): ?ClientPetEntity
    {
        return $this->clientPet;
    }

    public function setClientPet(?ClientPetEntity $clientPet): self
    {
        $this->clientPet = $clientPet;

        return $this;
    }

    /**
     * @return Collection<int, GroomingServiceEntity>
     */
    public function getGroomingServices(): Collection
    {
        return $this->groomingServices;
    }

    public function addGroomingService(GroomingServiceEntity $groomingService): self
    {
        if (!$this->groomingServices->contains($groomingService)) {
            $this->groomingServices[] = $groomingService;
            $groomingService->setGrooming($this);
        }

        return $this;
    }

    public function removeGroomingService(GroomingServiceEntity $groomingService): self
    {
        if ($this->groomingServices->removeElement($groomingService)) {
            // set the owning side to null (unless already changed)
            if ($groomingService->getGrooming() === $this) {
                $groomingService->setGrooming(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, GroomingPaymentEntity>
     */
    public function getGroomingPayments(): Collection
    {
        return $this->groomingPayments;
    }

    public function addGroomingPayment(GroomingPaymentEntity $groomingPayment): self
    {
        if (!$this->groomingPayments->contains($groomingPayment)) {
            $this->groomingPayments[] = $groomingPayment;
            $groomingPayment->setGrooming($this);
        }

        return $this;
    }

    public function removeGroomingPayment(GroomingPaymentEntity $groomingPayment): self
    {
        if ($this->groomingPayments->removeElement($groomingPayment)) {
            // set the owning side to null (unless already changed)
            if ($groomingPayment->getGrooming() === $this) {
                $groomingPayment->setGrooming(null);
            }
        }

        return $this;
    }

    public function getBilledAmount(): ?string
    {
        return $this->billedAmount;
    }

    public function setBilledAmount(?string $billedAmount): self
    {
        $this->billedAmount = $billedAmount;

        return $this;
    }

    public function isIsReadyForPickUp(): ?bool
    {
        return $this->isReadyForPickUp;
    }

    public function setIsReadyForPickUp(?bool $isReadyForPickUp): self
    {
        $this->isReadyForPickUp = $isReadyForPickUp;

        return $this;
    }

    public function getRemarks(): ?string
    {
        return $this->remarks;
    }

    public function setRemarks(?string $remarks): self
    {
        $this->remarks = $remarks;

        return $this;
    }

    public function getFileDescription(): ?string
    {
        return $this->fileDescription;
    }

    public function setFileDescription(?string $fileDescription): self
    {
        $this->fileDescription = $fileDescription;

        return $this;
    }

    public function getParsedFileDescription(): ?string
    {
        return $this->parsedFileDescription;
    }

    public function setParsedFileDescription(?string $parsedFileDescription): self
    {
        $this->parsedFileDescription = $parsedFileDescription;

        return $this;
    }

    /**
     * @return Collection<int, GroomingPhotoEntity>
     */
    public function getGroomingPhotos(): Collection
    {
        return $this->groomingPhotos;
    }

    public function addGroomingPhoto(GroomingPhotoEntity $groomingPhoto): self
    {
        if (!$this->groomingPhotos->contains($groomingPhoto)) {
            $this->groomingPhotos[] = $groomingPhoto;
            $groomingPhoto->setGrooming($this);
        }

        return $this;
    }

    public function removeGroomingPhoto(GroomingPhotoEntity $groomingPhoto): self
    {
        if ($this->groomingPhotos->removeElement($groomingPhoto)) {
            // set the owning side to null (unless already changed)
            if ($groomingPhoto->getGrooming() === $this) {
                $groomingPhoto->setGrooming(null);
            }
        }

        return $this;
    }

    public function getTransactionNo(): ?string
    {
        return $this->transactionNo;
    }

    public function setTransactionNo(string $transactionNo): self
    {
        $this->transactionNo = $transactionNo;

        return $this;
    }

    
}

