<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\GroomingServiceRepository")
 * @ORM\Table(name="grooming_service")
 * @ORM\HasLifecycleCallbacks()
 */

class GroomingServiceEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="status", type="string")
     */
    protected $status;

    /**
     * @ORM\Column(name="remarks", type="text" , nullable=true)
     */
    protected $remarks;

    /**
     * @ORM\Column(name="service_rate", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $serviceRate;
    
    /**
     * @ORM\ManyToOne(targetEntity="GroomingEntity", inversedBy="groomingServices")
     * @ORM\JoinColumn(name="grooming_id", referencedColumnName="id", nullable=true)
     */
    protected $grooming;

    /**
     * @ORM\ManyToOne(targetEntity="ServiceEntity", inversedBy="groomingServices")
     * @ORM\JoinColumn(name="service_id", referencedColumnName="id", nullable=true)
     */
    protected $service;

    /**
     * @ORM\ManyToOne(targetEntity="UserEntity", inversedBy="groomingServices")
     * @ORM\JoinColumn(name="user_id", referencedColumnName="id", nullable=true)
     */
    protected $user;

    
         /**
     * @ORM\OneToMany(targetEntity="CancelledServiceItemEntity", mappedBy="groomingService", cascade={"remove"})
     */
    protected $cancelledServiceItems;

    public function __construct($data = null)
    {
        $this->cancelledServiceItems = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return GroomingServiceEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getGrooming(): ?GroomingEntity
    {
        return $this->grooming;
    }

    public function setGrooming(?GroomingEntity $grooming): self
    {
        $this->grooming = $grooming;

        return $this;
    }

    public function getServiceRate(): ?string
    {
        return $this->serviceRate;
    }

    public function setServiceRate(?string $serviceRate): self
    {
        $this->serviceRate = str_replace(',','', $serviceRate);

        return $this;
    }

    public function getService(): ?ServiceEntity
    {
        return $this->service;
    }

    public function setService(?ServiceEntity $service): self
    {
        $this->service = $service;

        return $this;
    }

    public function getUser(): ?UserEntity
    {
        return $this->user;
    }

    public function setUser(?UserEntity $user): self
    {
        $this->user = $user;

        return $this;
    }

    public function getRemarks(): ?string
    {
        return $this->remarks;
    }

    public function setRemarks(?string $remarks): self
    {
        $this->remarks = $remarks;

        return $this;
    }

    /**
     * @return Collection<int, CancelledServiceItemEntity>
     */
    public function getCancelledServiceItems(): Collection
    {
        return $this->cancelledServiceItems;
    }

    public function addCancelledServiceItem(CancelledServiceItemEntity $cancelledServiceItem): self
    {
        if (!$this->cancelledServiceItems->contains($cancelledServiceItem)) {
            $this->cancelledServiceItems[] = $cancelledServiceItem;
            $cancelledServiceItem->setGroomingService($this);
        }

        return $this;
    }

    public function removeCancelledServiceItem(CancelledServiceItemEntity $cancelledServiceItem): self
    {
        if ($this->cancelledServiceItems->removeElement($cancelledServiceItem)) {
            // set the owning side to null (unless already changed)
            if ($cancelledServiceItem->getGroomingService() === $this) {
                $cancelledServiceItem->setGroomingService(null);
            }
        }

        return $this;
    }

  
   
}

