<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\InventoryRepository")
 * @ORM\Table(name="inventory")
 * @ORM\HasLifecycleCallbacks()
 */

class InventoryEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="beginning_quantity", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $beginningQuantity;
    
    /**
     * @ORM\Column(name="ending_quantity", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $endingQuantity;

    /**
     * @ORM\Column(name="buying_price", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $buyingPrice;

    /**
     * @ORM\Column(name="selling_price", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $sellingPrice;
    
    /**
     * @ORM\OneToOne(targetEntity="ItemEntity", inversedBy="inventory")
     * @ORM\JoinColumn(name="item_id", referencedColumnName="id", nullable=true)
     */
    protected $item;
        
    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="inventories")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\OneToMany(targetEntity="InventoryAdjustmentEntity", mappedBy="inventory", cascade={"remove"})
     */
    protected $inventoryAdjustments;

    public function __construct($data = null)
    {
        $this->inventoryAdjustments = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return InventoryEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }



    public function getBuyingPrice(): ?string
    {
        return $this->buyingPrice;
    }

    public function setBuyingPrice(?string $buyingPrice): self
    {
        $this->buyingPrice = str_replace(',', '', $buyingPrice);

        return $this;
    }

    public function getSellingPrice(): ?string
    {
        return $this->sellingPrice;
    }

    public function setSellingPrice(?string $sellingPrice): self
    {
        $this->sellingPrice = str_replace(',', '', $sellingPrice);

        return $this;
    }

    public function getItem(): ?ItemEntity
    {
        return $this->item;
    }

    public function setItem(?ItemEntity $item): self
    {
        $this->item = $item;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    public function getBeginningQuantity(): ?string
    {
        return $this->beginningQuantity;
    }

    public function setBeginningQuantity(?string $beginningQuantity): self
    {
        $this->beginningQuantity = str_replace(',', '', $beginningQuantity);

        return $this;
    }

    public function getEndingQuantity(): ?string
    {
        return $this->endingQuantity;
    }

    public function setEndingQuantity(?string $endingQuantity): self
    {
        $this->endingQuantity = str_replace(',', '', $endingQuantity);

        return $this;
    }

    /**
     * @return Collection<int, InventoryAdjustmentEntity>
     */
    public function getInventoryAdjustments(): Collection
    {
        return $this->inventoryAdjustments;
    }

    public function addInventoryAdjustment(InventoryAdjustmentEntity $inventoryAdjustment): self
    {
        if (!$this->inventoryAdjustments->contains($inventoryAdjustment)) {
            $this->inventoryAdjustments[] = $inventoryAdjustment;
            $inventoryAdjustment->setInventory($this);
        }

        return $this;
    }

    public function removeInventoryAdjustment(InventoryAdjustmentEntity $inventoryAdjustment): self
    {
        if ($this->inventoryAdjustments->removeElement($inventoryAdjustment)) {
            // set the owning side to null (unless already changed)
            if ($inventoryAdjustment->getInventory() === $this) {
                $inventoryAdjustment->setInventory(null);
            }
        }

        return $this;
    }

   
   
}

