<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\ItemRepository")
 * @ORM\Table(name="item")
 * @ORM\HasLifecycleCallbacks()
 */

class ItemEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="code", type="string")
     */
    protected $code;

    /**
     * @ORM\Column(name="description", type="string")
     */
    protected $description;
    
    /**
     * @ORM\OneToOne(targetEntity="InventoryEntity", mappedBy="item", cascade={"remove"})
     */
    protected $inventory;

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="items")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    
    /**
     * @ORM\OneToMany(targetEntity="ServiceItemEntity", mappedBy="item", cascade={"remove"})
     */
    protected $serviceItems;


    public function __construct($data = null)
    {
        $this->serviceItems = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return ItemEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    public function getInventory(): ?InventoryEntity
    {
        return $this->inventory;
    }

    public function setInventory(?InventoryEntity $inventory): self
    {
        // unset the owning side of the relation if necessary
        if ($inventory === null && $this->inventory !== null) {
            $this->inventory->setItem(null);
        }

        // set the owning side of the relation if necessary
        if ($inventory !== null && $inventory->getItem() !== $this) {
            $inventory->setItem($this);
        }

        $this->inventory = $inventory;

        return $this;
    }

    /**
     * @return Collection<int, ServiceItemEntity>
     */
    public function getServiceItems(): Collection
    {
        return $this->serviceItems;
    }

    public function addServiceItem(ServiceItemEntity $serviceItem): self
    {
        if (!$this->serviceItems->contains($serviceItem)) {
            $this->serviceItems[] = $serviceItem;
            $serviceItem->setItem($this);
        }

        return $this;
    }

    public function removeServiceItem(ServiceItemEntity $serviceItem): self
    {
        if ($this->serviceItems->removeElement($serviceItem)) {
            // set the owning side to null (unless already changed)
            if ($serviceItem->getItem() === $this) {
                $serviceItem->setItem(null);
            }
        }

        return $this;
    }

   

   
}

