<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\LodgeRepository")
 * @ORM\Table(name="lodge")
 * @ORM\HasLifecycleCallbacks()
 */

class LodgeEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="transaction_no", type="string")
     */
    protected $transactionNo;

    /**
     * @ORM\Column(name="status", type="string")
     */
    protected $status;

    /**
     * @ORM\Column(name="time_in", type="datetime", nullable=false)
     */
    protected $timeIn;

    /**
     * @ORM\Column(name="time_out", type="datetime", nullable=true)
     */
    protected $timeOut;

     /**
     * @ORM\Column(name="billed_amount", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $billedAmount;

     /**
     * @ORM\Column(name="room_rate", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $roomRate;

    /**
     * @ORM\Column(name="is_ready_for_pick_up", type="boolean", nullable=true)
     */
    protected $isReadyForPickUp;

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="lodges")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\ManyToOne(targetEntity="RoomEntity", inversedBy="lodges")
     * @ORM\JoinColumn(name="room_id", referencedColumnName="id", nullable=true)
     */
    protected $room;

    /**
     * @ORM\ManyToOne(targetEntity="ClientPetEntity", inversedBy="lodges")
     * @ORM\JoinColumn(name="client_pet_id", referencedColumnName="id", nullable=true)
     */
    protected $clientPet;

    /**
     * @ORM\OneToMany(targetEntity="LodgeServiceEntity", mappedBy="lodge", cascade={"remove"})
     */
    protected $lodgeServices;

    /**
     * @ORM\OneToMany(targetEntity="LodgePaymentEntity", mappedBy="lodge", cascade={"remove"})
     */
    protected $lodgePayments;

        /**
     * @ORM\OneToMany(targetEntity="BranchSmsEntity", mappedBy="lodge", cascade={"remove"})
     */
    protected $branchSmss;



    
    
    public function __construct($data = null)
    {
        $this->lodgeServices = new ArrayCollection();
        $this->lodgePayments = new ArrayCollection();
        $this->branchSmss = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return LodgeEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

     /**
     * Get totalServiceCharge
     *
     *
     * @return decimal
     */
    public function getTotalServiceCharge()
    {
        $mt = 0;

        foreach($this->getLodgeServices() as $service){
            
            if(!$service->getIsDeleted()){
                $mt+= $service->getServiceRate();
            }
        }

        return $mt;
    }

     /**
     * Get totalPayment
     *
     *
     * @return decimal
     */
    public function getTotalPayment()
    {
        $mt = 0;

        foreach($this->getLodgePayments() as $p){
            
            if(!$p->getIsDeleted()){
                $mt+= $p->getAmount();
            }
        }

        return $mt;
    }

    

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getStatus(): ?string
    {
        return $this->status;
    }

    public function setStatus(string $status): self
    {
        $this->status = $status;

        return $this;
    }

    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    public function getRoom(): ?RoomEntity
    {
        return $this->room;
    }

    public function setRoom(?RoomEntity $room): self
    {
        $this->room = $room;

        return $this;
    }

    public function getClientPet(): ?ClientPetEntity
    {
        return $this->clientPet;
    }

    public function setClientPet(?ClientPetEntity $clientPet): self
    {
        $this->clientPet = $clientPet;

        return $this;
    }

    public function getTimeIn(): ?\DateTimeInterface
    {
        return $this->timeIn;
    }

    public function setTimeIn(\DateTimeInterface $timeIn): self
    {
        $this->timeIn = $timeIn;

        return $this;
    }

    public function getTimeOut(): ?\DateTimeInterface
    {
        return $this->timeOut;
    }

    public function setTimeOut(?\DateTimeInterface $timeOut): self
    {
        $this->timeOut = $timeOut;

        return $this;
    }

    /**
     * @return Collection<int, LodgeServiceEntity>
     */
    public function getLodgeServices(): Collection
    {
        return $this->lodgeServices;
    }

    public function addLodgeService(LodgeServiceEntity $lodgeService): self
    {
        if (!$this->lodgeServices->contains($lodgeService)) {
            $this->lodgeServices[] = $lodgeService;
            $lodgeService->setLodge($this);
        }

        return $this;
    }

    public function removeLodgeService(LodgeServiceEntity $lodgeService): self
    {
        if ($this->lodgeServices->removeElement($lodgeService)) {
            // set the owning side to null (unless already changed)
            if ($lodgeService->getLodge() === $this) {
                $lodgeService->setLodge(null);
            }
        }

        return $this;
    }

    public function getBilledAmount(): ?string
    {
        return $this->billedAmount;
    }

    public function setBilledAmount(?string $billedAmount): self
    {
        $this->billedAmount = $billedAmount;

        return $this;
    }

    /**
     * @return Collection<int, LodgePaymentEntity>
     */
    public function getLodgePayments(): Collection
    {
        return $this->lodgePayments;
    }

    public function addLodgePayment(LodgePaymentEntity $lodgePayment): self
    {
        if (!$this->lodgePayments->contains($lodgePayment)) {
            $this->lodgePayments[] = $lodgePayment;
            $lodgePayment->setLodge($this);
        }

        return $this;
    }

    public function removeLodgePayment(LodgePaymentEntity $lodgePayment): self
    {
        if ($this->lodgePayments->removeElement($lodgePayment)) {
            // set the owning side to null (unless already changed)
            if ($lodgePayment->getLodge() === $this) {
                $lodgePayment->setLodge(null);
            }
        }

        return $this;
    }

    public function isIsReadyForPickUp(): ?bool
    {
        return $this->isReadyForPickUp;
    }

    public function setIsReadyForPickUp(?bool $isReadyForPickUp): self
    {
        $this->isReadyForPickUp = $isReadyForPickUp;

        return $this;
    }

    /**
     * @return Collection<int, BranchSmsEntity>
     */
    public function getBranchSmss(): Collection
    {
        return $this->branchSmss;
    }

    public function addBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if (!$this->branchSmss->contains($branchSmss)) {
            $this->branchSmss[] = $branchSmss;
            $branchSmss->setLodge($this);
        }

        return $this;
    }

    public function removeBranchSmss(BranchSmsEntity $branchSmss): self
    {
        if ($this->branchSmss->removeElement($branchSmss)) {
            // set the owning side to null (unless already changed)
            if ($branchSmss->getLodge() === $this) {
                $branchSmss->setLodge(null);
            }
        }

        return $this;
    }

    public function getTransactionNo(): ?string
    {
        return $this->transactionNo;
    }

    public function setTransactionNo(string $transactionNo): self
    {
        $this->transactionNo = $transactionNo;

        return $this;
    }

    public function getRoomRate(): ?string
    {
        return $this->roomRate;
    }

    public function setRoomRate(?string $roomRate): self
    {
        $this->roomRate = $roomRate;

        return $this;
    }

   
   
}

