<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\ServiceRepository")
 * @ORM\Table(name="service")
 * @ORM\HasLifecycleCallbacks()
 */

class ServiceEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="code", type="string")
     */
    protected $code;

    /**
     * @ORM\Column(name="description", type="string")
     */
    protected $description;

    /**
     * @ORM\Column(name="service_rate", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $serviceRate;
    

    /**
     * @ORM\ManyToOne(targetEntity="BranchEntity", inversedBy="services")
     * @ORM\JoinColumn(name="branch_id", referencedColumnName="id", nullable=true)
     */
    protected $branch;

    /**
     * @ORM\ManyToOne(targetEntity="ServiceTypeEntity", inversedBy="services")
     * @ORM\JoinColumn(name="service_type_id", referencedColumnName="id", nullable=true)
     */
    protected $serviceType;

    /**
     * @ORM\OneToMany(targetEntity="LodgeServiceEntity", mappedBy="service", cascade={"remove"})
     */
    protected $lodgeServices;

        /**
     * @ORM\OneToMany(targetEntity="ServiceItemEntity", mappedBy="service", cascade={"remove"})
     */
    protected $serviceItems;

    
    public function __construct($data = null)
    {
        $this->lodgeServices = new ArrayCollection();
        $this->serviceItems = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return ServiceEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getCode(): ?string
    {
        return $this->code;
    }

    public function setCode(string $code): self
    {
        $this->code = $code;

        return $this;
    }

    public function getDescription(): ?string
    {
        return $this->description;
    }

    public function setDescription(string $description): self
    {
        $this->description = $description;

        return $this;
    }

    public function getServiceRate(): ?string
    {
        return $this->serviceRate;
    }

    public function setServiceRate(?string $serviceRate): self
    {
        $this->serviceRate = str_replace(',','', $serviceRate);

        return $this;
    }


    public function getBranch(): ?BranchEntity
    {
        return $this->branch;
    }

    public function setBranch(?BranchEntity $branch): self
    {
        $this->branch = $branch;

        return $this;
    }

    public function getServiceType(): ?ServiceTypeEntity
    {
        return $this->serviceType;
    }

    public function setServiceType(?ServiceTypeEntity $serviceType): self
    {
        $this->serviceType = $serviceType;

        return $this;
    }

    /**
     * @return Collection<int, LodgeServiceEntity>
     */
    public function getLodgeServices(): Collection
    {
        return $this->lodgeServices;
    }

    public function addLodgeService(LodgeServiceEntity $lodgeService): self
    {
        if (!$this->lodgeServices->contains($lodgeService)) {
            $this->lodgeServices[] = $lodgeService;
            $lodgeService->setService($this);
        }

        return $this;
    }

    public function removeLodgeService(LodgeServiceEntity $lodgeService): self
    {
        if ($this->lodgeServices->removeElement($lodgeService)) {
            // set the owning side to null (unless already changed)
            if ($lodgeService->getService() === $this) {
                $lodgeService->setService(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, ServiceItemEntity>
     */
    public function getServiceItems(): Collection
    {
        return $this->serviceItems;
    }

    public function addServiceItem(ServiceItemEntity $serviceItem): self
    {
        if (!$this->serviceItems->contains($serviceItem)) {
            $this->serviceItems[] = $serviceItem;
            $serviceItem->setService($this);
        }

        return $this;
    }

    public function removeServiceItem(ServiceItemEntity $serviceItem): self
    {
        if ($this->serviceItems->removeElement($serviceItem)) {
            // set the owning side to null (unless already changed)
            if ($serviceItem->getService() === $this) {
                $serviceItem->setService(null);
            }
        }

        return $this;
    }

  

   
}

