<?php

namespace App\Entity;

use App\Entity\Base\BaseEntity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Form\Extension\Core\DataTransformer\DateTimeToStringTransformer;

/**
 * @ORM\Entity(repositoryClass="App\Repository\ServiceItemRepository")
 * @ORM\Table(name="service_item")
 * @ORM\HasLifecycleCallbacks()
 */

class ServiceItemEntity extends BaseEntity
{
    /**
     * @ORM\Column(type="bigint")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected $id;

    /**
     * @ORM\Column(name="quantity", type="decimal", precision=12, scale=2, nullable=true)
     */
    protected $quantity;
    
    /**
     * @ORM\ManyToOne(targetEntity="ServiceEntity", inversedBy="serviceItems")
     * @ORM\JoinColumn(name="service_id", referencedColumnName="id", nullable=true)
     */
    protected $service;
    
    /**
     * @ORM\ManyToOne(targetEntity="ItemEntity", inversedBy="serviceItems")
     * @ORM\JoinColumn(name="item_id", referencedColumnName="id", nullable=true)
     */
    protected $item;

      /**
     * @ORM\OneToMany(targetEntity="UseServiceItemEntity", mappedBy="serviceItem", cascade={"remove"})
     */
    protected $useServiceItems;

    /**
     * @ORM\OneToMany(targetEntity="CancelledServiceItemEntity", mappedBy="serviceItem", cascade={"remove"})
     */
    protected $cancelledServiceItems;

    public function __construct($data = null)
    {
        $this->useServiceItems = new ArrayCollection();
        $this->cancelledServiceItems = new ArrayCollection();
    }


    /**
     * Set isDeleted
     *
     * @param boolean $isDeleted
     *
     * @return ServiceItemEntity
     */
    public function setIsDeleted($isDeleted)
    {
        $this->isDeleted = $isDeleted;

        return $this;
    }

    /*--------------------------------------------------------------------------------------------------------*/
    /*					User Defined Setters and Getters													  */
    /*--------------------------------------------------------------------------------------------------------*/

    public function getId(): ?string
    {
        return $this->id;
    }

    public function getQuantity(): ?string
    {
        return $this->quantity;
    }

    public function setQuantity(?string $quantity): self
    {
        $this->quantity = str_replace(',','', $quantity);

        return $this;
    }

    public function getService(): ?ServiceEntity
    {
        return $this->service;
    }

    public function setService(?ServiceEntity $service): self
    {
        $this->service = $service;

        return $this;
    }

    public function getItem(): ?ItemEntity
    {
        return $this->item;
    }

    public function setItem(?ItemEntity $item): self
    {
        $this->item = $item;

        return $this;
    }

    /**
     * @return Collection<int, UseServiceItemEntity>
     */
    public function getUseServiceItems(): Collection
    {
        return $this->useServiceItems;
    }

    public function addUseServiceItem(UseServiceItemEntity $useServiceItem): self
    {
        if (!$this->useServiceItems->contains($useServiceItem)) {
            $this->useServiceItems[] = $useServiceItem;
            $useServiceItem->setServiceItem($this);
        }

        return $this;
    }

    public function removeUseServiceItem(UseServiceItemEntity $useServiceItem): self
    {
        if ($this->useServiceItems->removeElement($useServiceItem)) {
            // set the owning side to null (unless already changed)
            if ($useServiceItem->getServiceItem() === $this) {
                $useServiceItem->setServiceItem(null);
            }
        }

        return $this;
    }

    /**
     * @return Collection<int, CancelledServiceItemEntity>
     */
    public function getCancelledServiceItems(): Collection
    {
        return $this->cancelledServiceItems;
    }

    public function addCancelledServiceItem(CancelledServiceItemEntity $cancelledServiceItem): self
    {
        if (!$this->cancelledServiceItems->contains($cancelledServiceItem)) {
            $this->cancelledServiceItems[] = $cancelledServiceItem;
            $cancelledServiceItem->setServiceItem($this);
        }

        return $this;
    }

    public function removeCancelledServiceItem(CancelledServiceItemEntity $cancelledServiceItem): self
    {
        if ($this->cancelledServiceItems->removeElement($cancelledServiceItem)) {
            // set the owning side to null (unless already changed)
            if ($cancelledServiceItem->getServiceItem() === $this) {
                $cancelledServiceItem->setServiceItem(null);
            }
        }

        return $this;
    }

  

   
}

