<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;


use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;

use App\Form\DataTransformer\DataTransformer;
use App\Form\DataTransformer\DatetimeTransformer;

use App\Entity\ScheduleEntity;
use App\Entity\BranchEntity;
use App\Entity\ClientPetEntity;

class ScheduleForm extends AbstractType
{
   
    private $manager;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)
        
            ->add('remarks', TextareaType::class, array(
                'label' => 'Remarks',
                'label_attr' => array(
                    'class' => 'middle required  col-form-label col-3'
                ),
                'attr' => array(
                    'class' => 'form-control '
                ),
                'required' => false
            ))
            ->add('status', ChoiceType::class, [
                'label' => 'status',
                'choices' => ['Confirmed - Show' => 'Confirmed - Show' , 'Confirmed - No Show' => 'Confirmed - No Show', 'Cancelled' => 'Cancelled', 'Declined' => 'Declined'],
                'required' => false, 'attr' => array(
                    'class' => 'form-control '
                ),
                'placeholder' => false,

            ])
            ->add('schedule_type', HiddenType::class, array('data' => $options['currentModule']))
            ->add('client_pet', HiddenType::class)
            ->add('branch', HiddenType::class, array('data' => $options['branchId']))
            
            ->addEventListener(FormEvents::PRE_SET_DATA, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();

                $cp = $data->getClientPet();
                $form
                    ->add('client_pet_desc', TextType::class, array(
                        'label' => 'Client',
                        'label_attr' => array(
                            'class' => 'middle required  col-form-label'
                        ),
                        'required' => true,
                        'attr' => array(                            
                            'class' => 'form-control ',
                        ),
                        'mapped' => false,
                        'data' => $cp ? $cp->getClient()->getFullName() . ' - ' . $cp->getPetName() : ''
                    ))
                    ->add('schedule_date', TextType::class, array(
                        'label' => 'Date',
                        'required' => true,
                        'label_attr' => array(
                            'class' => 'middle required'
                        ),
                        'attr' => [ 'class' => 'form-control required datepicker'],
                        'mapped' => true,
                        'data' => $data->getScheduleDate() ? $data->getScheduleDate()->format('m/d/Y')  : date('m/d/Y')

                    ))
                    ->add('schedule_time', TextType::class, array(
                        'label' => 'Time',
                        'required' => true,
                        'label_attr' => array(
                            'class' => 'middle '
                        ),
                        'attr' => [ 'class' => 'form-control required timepicker'],
                        'data' => $data->getScheduleDate() ? $data->getScheduleDate()->format('h:i A')  : date('h:i A'),
                        'mapped' => false
                    ));
                   
           
                 
            })
            ->addEventListener(FormEvents::PRE_SUBMIT, function(FormEvent $event, $options) {

                $form = $event->getForm();
                $data = $event->getData();

                $date = $data['schedule_date'];
                $data['schedule_date'] = !empty($date) && !empty($data['schedule_time']) ?  new \Datetime($date . ' ' . $data['schedule_time']) : null;
                $event->setData($data);
                
                 
            });
               
            $builder->get('branch')->addModelTransformer(new DataTransformer($this->manager, BranchEntity::class, true, $options['branchId']));
            $builder->get('client_pet')->addModelTransformer(new DataTransformer($this->manager, ClientPetEntity::class, false));




    }

    public function getName()
    {
        return 'schedule';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\ScheduleEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'branchId'    => null,
            'currentModule' => '' 
        ));
    }
}