<?php

namespace App\Form;

use Doctrine\Persistence\ObjectManager;
use Doctrine\ORM\EntityRepository;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\PasswordType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\HiddenType;
use Symfony\Component\Form\Extension\Core\Type\RepeatedType;
use Symfony\Component\Form\Extension\Core\Type\EmailType;
use Symfony\Component\Form\Extension\Core\Type\CheckboxType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\DateType;
use Symfony\Component\Form\Extension\Core\Type\NumberType;


use Symfony\Component\Form\FormEvents;
use Symfony\Component\Form\FormEvent;

use App\Form\DataTransformer\DataTransformer;
use App\Entity\SmsEntity;
use App\Entity\CompanyEntity;

class SmsForm extends AbstractType
{
   
    private $manager;

    public function __construct(ObjectManager $manager) {
        $this->manager = $manager;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $builder
            ->add('action', HiddenType::class, array(
                'data' => $options['action'],
                'mapped' => false,
                'attr' => array(
                    'class' => 'form-action'
                )
            ))
            ->add('id', HiddenType::class)
            ->add('message', TextareaType::class, array(
                'label' => 'Message',
                'label_attr' => array(
                    'class' => 'middle'
                ),
                'attr' => [ 'class' => 'form-control', 'rows' => 10],
                'required' => false
            ))
            ->add('is_lodge_ready_for_pick_up', CheckboxType::class, [
                'label'    => 'Pet is ready for pickup',
                'required' => false,
                'label_attr' => ['class' => 'form-check-label']
            ])
            ->add('is_service_on_process', CheckboxType::class, [
                'label'    => 'Service is on process',
                'required' => false,
                'label_attr' => ['class' => 'form-check-label']
            ])
            ->add('is_service_completed', CheckboxType::class, [
                'label'    => 'Service is completed',
                'required' => false,
                'label_attr' => ['class' => 'form-check-label']

            ])
            ->add('company', HiddenType::class, array('data' => $options['companyId']));    
            $builder->get('company')->addModelTransformer(new DataTransformer($this->manager, CompanyEntity::class, true, $options['companyId']));


    }

    public function getName()
    {
        return 'sms';
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(array(
            'data_class' 	  => 'App\Entity\SmsEntity',
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            // a unique key to help generate the secret token
            'intention'       => 'clientEntity_intention',
            'action'          => 'n',
            'companyId'    => null 
        ));
    }
}