<?php

namespace App\Repository;

use App\Entity\InventoryEntity;
use App\Entity\BranchEntity;


/**
 * InventoryRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class InventoryRepository extends \Doctrine\ORM\EntityRepository
{
    public function validate($form) {

        $errors = array();


        $action = $form['action'];

        // d = delete
        if($action !== 'd') {

            
            $inventoryExist = $this->getEntityManager()->getRepository(InventoryEntity::class)
                ->createQueryBuilder('c')
                ->where('c.id != :id')
                ->andWhere('c.item = :item')
                ->andWhere('c.isDeleted = :is_deleted')
                ->andWhere('c.branch = :branch')
                ->setParameters(array(
                    'id' => $form['id'],
                    'item' => $form['item'],
                    'is_deleted' => false,
                    'branch' => base64_decode($form['branch'])
                ))
                ->getQuery()->getResult();
            
            if($action != 'u' && $inventoryExist){
                $errors[] = 'Item has already listed in inventory';
            }

            if(empty($form['beginning_quantity'])){
                $errors[] = 'Beginning Quantity should not be blank.';
            }
          
        }

        return $errors;
    }

    public function ajax_list(array $get, array $userData){

        $columns = array(
            array('i.`description`', 'i.`description`', 'item'),
            array('c.`buying_price`', 'c.`buying_price`', 'buyingPrice'),
            array('c.`selling_price`', 'c.`selling_price`', 'sellingPrice'),
            array('c.`beginning_quantity`', 'c.`beginning_quantity`', 'beginningQuantity'),
            array('c.`ending_quantity`', 'c.`ending_quantity`', 'endingQuantity'),
            array('c.`id`', "c.`id`")
        );
        $asColumns = array();

        $select = "SELECT";
        $from = "FROM `inventory` c";
        $sqlWhere = "WHERE c.`is_deleted` = 0";
        $joins = "LEFT JOIN `item` i ON  i.`id` = c.`item_id` ";
        $groupBy = "";
        $orderBy = "";
        $limit = "";
        $stmtParams = array();

        if($userData['type'] != 'Super Admin'){

            $sqlWhere .= " AND c.`branch_id` = :branchId";
            $stmtParams['branchId'] =  base64_decode($userData['branchId']);
        }

        foreach($columns as $key => $column) {
            $select .= ($key > 0 ? ', ' : ' ') . $column[1] . (isset($column[2]) ? ' AS ' . $column[2] : '');
        }

        /*
         * Ordering
         */
        foreach($get['columns'] as $key => $column) {
            if($column['orderable']=='true') {
                if(isSet($get['order'])) {
                    foreach($get['order'] as $order) {
                        if($order['column']==$key) {
                            $orderBy .= (!empty($orderBy) ? ', ' : 'ORDER BY ') . $columns[$key][0] . (!empty($order['dir']) ? ' ' . $order['dir'] : '');
                        }
                    }
                }
            }
        }

        /*
         * Filtering
         */
        if(isset($get['search']) && $get['search']['value'] != ''){
            $aLikes = array();
            foreach($get['columns'] as $key => $column) {
                if($column['searchable']=='true') {
                    $aLikes[] = $columns[$key][0] . ' LIKE :searchValue';
                }
            }
            foreach($asColumns as $asColumn) {
                $aLikes[] = $asColumn . ' LIKE :searchValue';
            }
            if(count($aLikes)) {
                $sqlWhere .= (!empty($sqlWhere) ? ' AND ' : 'WHERE ') . '(' . implode(' OR ', $aLikes) . ')';
                $stmtParams['searchValue'] = "%" . $get['search']['value'] . "%";
            }
        }

        /* Set Limit and Length */
        if(isset( $get['start'] ) && $get['length'] != '-1'){
            $limit = 'LIMIT ' . (int)$get['start'] . ',' . (int)$get['length'];
        }

        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result_count = $res->fetchAllAssociative();




        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy $limit";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();

        /* Data Count */
        $recordsTotal = count($result_count);


              /*
         * Output
         */
        $output = array(
            "draw" => intval($get['draw']),
            "recordsTotal" => $recordsTotal,
            "recordsFiltered" => $recordsTotal,
            "data" => array()
        );

        $url = $get['url'];
        $formUrl = '';
        $hasDetails = false;

        if($userData['type'] == 'Super Admin'|| in_array('Inventory Details', $userData['accesses'])){

            $formUrl = $url . '/inventory/details';  
            $hasDetails = true;
        }

        foreach($result as $row) {

            $id = base64_encode($row['id']);

            $action = $hasDetails ? '<a href="'.$formUrl.'/'.$id.'" class="href-modal action-button-style btn btn-primary table-btn" >Details</a>' : "";

            $values = array(
                $row['item'],
                $row['buyingPrice'],
                $row['sellingPrice'],
                $row['beginningQuantity'],
                $row['endingQuantity'],
                $action
            );

            $output['data'][] = $values;
        }

        unset($result);

        return $output;
    }

    public function autocompleteSuggestions($q, array $userData) {
       
        $stmtParams = array();

        $qs = $q['query'];

        $where = ' WHERE b.`description` LIKE :description';
        $stmtParams['description'] =  "%$qs%"; 

        if($userData['type'] != 'Super Admin' ){

            $where.= ' AND b.`branch_id` = :branchId';
            $stmtParams['branchId'] = base64_decode($userData['branchId']);
        }
      
        $sql = "
            SELECT
                 b.`id`,
                 b.`description` AS data,
                 b.`description` AS value
            FROM `inventory` b
            ". $where ."
            AND b.`is_deleted` = 0
            ORDER BY b.`description`
            LIMIT 0,20
        ";


        $query = $this->getEntityManager()->getConnection()->prepare($sql);

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
       
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();

        return $result;
    }

   
  

}
