<?php

namespace App\Repository;

use App\Entity\RoomEntity;
use App\Entity\BranchEntity;


/**
 * RoomRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class RoomRepository extends \Doctrine\ORM\EntityRepository
{
    public function validate($form) {

        $errors = array();


        $action = $form['action'];

        // d = delete
        if($action !== 'd') {

            
            $roomExist = $this->getEntityManager()->getRepository(RoomEntity::class)
                ->createQueryBuilder('c')
                ->where('c.id != :id')
                ->andWhere('c.roomType = :roomType')
                ->andWhere('c.description = :description')
                ->andWhere('c.isDeleted = :is_deleted')
                ->andWhere('c.branch = :branch')
                ->setParameters(array(
                    'id' => $form['id'],
                    'description' => $form['description'],
                    'is_deleted' => false,
                    'roomType' => $form['room_type'],
                    'branch' => base64_decode($form['branch'])
                ))
                ->getQuery()->getResult();
            
            if($action != 'u' && $roomExist){
                $errors[] = 'Room already exist.';
            }

            if(empty($form['description'])){
                $errors[] = 'Description should not be blank.';
            }

            if(empty($form['minimum_hour'])){
                $errors[] = 'Minimum Hour should not be blank.';
            }

            if(empty($form['rate_per_minimum_hour'])){
                $errors[] = 'Rate Per Minimum hour should not be blank.';
            }

            if(empty($form['room_type'])){
                $errors[] = 'Room Type should not be blank.';
            }

          
        }

        return $errors;
    }

    public function ajax_list(array $get, array $userData){

        $columns = array(
            array('rr.`description`','rr.`description`', 'roomType'),
            array('c.`code`', 'c.`code`'),
            array('c.`description`', 'c.`description`'),
            array('c.`rate_per_minimum_hour`', 'c.`rate_per_minimum_hour`', 'roomRate'),
            array('c.`status`', 'c.`status`', 'status'),
            array('c.`id`', "c.`id`")
        );
        $asColumns = array();

        $select = "SELECT";
        $from = "FROM `room` c";
        $joins = "LEFT JOIN `room_type` rr ON rr.`id` = c.`room_type_id`";
        $sqlWhere = "WHERE c.`is_deleted` = 0";
        $groupBy = "";
        $orderBy = "";
        $limit = "";
        $stmtParams = array();

        if($userData['type'] != 'Super Admin'){

            $sqlWhere .= " AND c.`branch_id` = :branchId";
            $stmtParams['branchId'] =  base64_decode($userData['branchId']);
        }

        foreach($columns as $key => $column) {
            $select .= ($key > 0 ? ', ' : ' ') . $column[1] . (isset($column[2]) ? ' AS ' . $column[2] : '');
        }

        /*
         * Ordering
         */
        foreach($get['columns'] as $key => $column) {
            if($column['orderable']=='true') {
                if(isSet($get['order'])) {
                    foreach($get['order'] as $order) {
                        if($order['column']==$key) {
                            $orderBy .= (!empty($orderBy) ? ', ' : 'ORDER BY ') . $columns[$key][0] . (!empty($order['dir']) ? ' ' . $order['dir'] : '');
                        }
                    }
                }
            }
        }

        /*
         * Filtering
         */
        if(isset($get['search']) && $get['search']['value'] != ''){
            $aLikes = array();
            foreach($get['columns'] as $key => $column) {
                if($column['searchable']=='true') {
                    $aLikes[] = $columns[$key][0] . ' LIKE :searchValue';
                }
            }
            foreach($asColumns as $asColumn) {
                $aLikes[] = $asColumn . ' LIKE :searchValue';
            }
            if(count($aLikes)) {
                $sqlWhere .= (!empty($sqlWhere) ? ' AND ' : 'WHERE ') . '(' . implode(' OR ', $aLikes) . ')';
                $stmtParams['searchValue'] = "%" . $get['search']['value'] . "%";
            }
        }

        /* Set Limit and Length */
        if(isset( $get['start'] ) && $get['length'] != '-1'){
            $limit = 'LIMIT ' . (int)$get['start'] . ',' . (int)$get['length'];
        }

        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result_count = $res->fetchAllAssociative();




        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy $limit";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();

        /* Data Count */
        $recordsTotal = count($result_count);


              /*
         * Output
         */
        $output = array(
            "draw" => intval($get['draw']),
            "recordsTotal" => $recordsTotal,
            "recordsFiltered" => $recordsTotal,
            "data" => array()
        );

        $url = $get['url'];
        $formUrl = '';
        $hasUpdate = false;

        if($userData['type'] == 'Super Admin'|| in_array('Room Update', $userData['accesses'])){

            $formUrl = $url . '/room/ajax_form';  
            $hasUpdate = true;
        }

        foreach($result as $row) {

            $id = base64_encode($row['id']);

            $action = $hasUpdate ? '<a href="javascript:void(0);" class="href-modal action-btn action-button-style btn btn-primary table-btn" data-id="'.$id.'"  data-url="'.$formUrl.'" data-action="u">Update</a>' : "";

            $values = array(
                $row['roomType'],
                $row['description'],
                $row['code'],
                $row['roomRate'],
                $row['status'],
                $action
            );

            $output['data'][] = $values;
        }

        unset($result);

        return $output;
    }

    public function autocomplete_suggestions($q, array $userData) {
       
        $stmtParams = array();

        $qs = $q['query'];

        $where = ' WHERE b.`description` LIKE :description';
        $stmtParams['description'] =  "%$qs%"; 

        if($userData['type'] != 'Super Admin' ){

            $where.= ' AND b.`branch_id` = :branchId';
            $stmtParams['branchId'] = base64_decode($userData['branchId']);
        }
      
        $sql = "
            SELECT
                 b.`id`,
                 b.`description` AS data,
                 b.`description` AS value
            FROM `room` b
            ". $where ."
            AND b.`is_deleted` = 0
            ORDER BY b.`description`
            LIMIT 0,20
        ";


        $query = $this->getEntityManager()->getConnection()->prepare($sql);

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
       
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();

        return $result;
    }

    public function getByBranch($user){
        
        $branchId = $user->getBranch()->getId();
      
        $sql = "
            SELECT
                r.`id`,
                r.`rate_per_minimum_hour` AS roomRate,
                rt.`description` AS roomType,
                r.status,
                r.`description` AS room,
                l.client AS roomClient
            FROM `room` r
            LEFT JOIN `room_type` rt ON rt.`id` = r.`room_type_id`
            LEFT JOIN (
                SELECT 
                    l.`room_id`,
                    CONCAT(c.`first_name`, ' ' , c.`last_name`) AS client
                FROM `lodge` l
                LEFT JOIN `client_pet` cp ON cp.`id` = l.`client_pet_id`
                LEFT JOIN  `client` c ON c.`id` = cp.`client_id`
                WHERE l.`status` = 'Active'
                LIMIT 1 
            ) l ON l.`room_id` = r.`id`
            WHERE r.`branch_id` = ".$branchId."
            AND r.`is_deleted` = 0
           
        ";


        $query = $this->getEntityManager()->getConnection()->prepare($sql);

       
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();
        return $result;

    }

  

}
