<?php

namespace App\Repository;

use App\Entity\UserEntity;

/**
 * UserRepository
 *
 * This class was generated by the Doctrine ORM. Add your own custom
 * repository methods below.
 */
class UserRepository extends \Doctrine\ORM\EntityRepository
{

    public function validate($user_form) {

        $errors = array();

        $action = $user_form['action'];

        // d = delete
        if($action !== 'd') {

            $userExist = $this->getEntityManager()->getRepository(UserEntity::class)
                ->createQueryBuilder('u')
                ->where('u.id != :id')
                ->andWhere('u.username = :username')
                ->setParameters(array(
                    'id' => $user_form['id'],
                    'username' => $user_form['username']
                ))
                ->getQuery()->getResult();

            if($userExist) {
                $errors[] = 'Username already exist.';
            }

            if(empty($user_form['branch'])) {
                $errors[] = 'Please select branch.';
            }

            if(empty($user_form['type'])) {
                $errors[] = 'Please select type.';
            }

            if(empty($user_form['username'])) {
                $errors[] = 'Username should not be blank.';
            } else {

                if(strlen($user_form['username']) < 3) {
                    $errors[] = 'Username must be at least 3 characters.';
                }
            }

            if($action === 'n' && empty($user_form['password']['first']) && empty($user_form['password']['second'])) {
                $errors[] = 'Please enter your password.';
            } else if($user_form['password']['first'] !== $user_form['password']['second']) {
                $errors[] = 'Password does not match.';
            }

            if(empty($user_form['first_name'])) {
                $errors[] = 'First name should not be blank.';
            }

            if(empty($user_form['last_name'])) {
                $errors[] = 'Last name should not be blank.';
            }
        }

        return $errors;
    }

    public function ajax_list(array $get, $userData){

        $columns = array(
            array('u.`username`', 'u.`username`'),
            array('u.`first_name`', "CONCAT(u.`first_name`, ' ', u.`last_name`)", 'fullName'),
            array('u.`type`', 'u.`type`'),
            array('b.`description`', 'b.`description`', 'branch'),
            array('u.`contact_no`', 'u.`contact_no`'),
            array('u.`id`', "u.`id`")
        );
        $asColumns = array();

        $select = "SELECT";
        $from = "FROM `user` u";
        $joins = "LEFT JOIN  branch b ON u.`branch_id` = b.`id`";
        $sqlWhere = "WHERE u.type != 'Super Admin'";
        $groupBy = "";
        $orderBy = "";
        $limit = "";
        $stmtParams = array();

        foreach($columns as $key => $column) {
            $select .= ($key > 0 ? ', ' : ' ') . $column[1] . (isset($column[2]) ? ' AS ' . $column[2] : '');
        }


        if($userData['type'] != 'Super Admin' || $get['companyId']){

            $sqlWhere .= " AND u.`company_id` = :companyId";
            $stmtParams['companyId'] = base64_decode($get['companyId']);
        }

        /*
         * Ordering
         */
        foreach($get['columns'] as $key => $column) {
            if($column['orderable']=='true') {
                if(isSet($get['order'])) {
                    foreach($get['order'] as $order) {
                        if($order['column']==$key) {
                            $orderBy .= (!empty($orderBy) ? ', ' : 'ORDER BY ') . $columns[$key][0] . (!empty($order['dir']) ? ' ' . $order['dir'] : '');
                        }
                    }
                }
            }
        }

        /*
         * Filtering
         */
        if(isset($get['search']) && $get['search']['value'] != ''){
            $aLikes = array();
            foreach($get['columns'] as $key => $column) {
                if($column['searchable']=='true') {
                    $aLikes[] = $columns[$key][0] . ' LIKE :searchValue';
                }
            }
            foreach($asColumns as $asColumn) {
                $aLikes[] = $asColumn . ' LIKE :searchValue';
            }
            if(count($aLikes)) {
                $sqlWhere .= (!empty($sqlWhere) ? ' AND ' : 'WHERE ') . '(' . implode(' OR ', $aLikes) . ')';
                $stmtParams['searchValue'] = "%" . $get['search']['value'] . "%";
            }
        }

        /* Set Limit and Length */
        if(isset( $get['start'] ) && $get['length'] != '-1'){
            $limit = 'LIMIT ' . (int)$get['start'] . ',' . (int)$get['length'];
        }

        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
     

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result_count = $res->fetchAllAssociative();




        $sql = "$select $from $joins $sqlWhere $groupBy $orderBy $limit";
        $query = $this->getEntityManager()->getConnection()->prepare($sql);
        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();

        /* Data Count */
        $recordsTotal = count($result_count);

        /*
         * Output
         */
        $output = array(
            "draw" => intval($get['draw']),
            "recordsTotal" => $recordsTotal,
            "recordsFiltered" => $recordsTotal,
            "data" => array()
        );

        $url = $get['url'];
        $formUrl = '';
        $hasUpdate = false;
        if($userData['type'] == 'Super Admin'  || in_array('Company View User Update', $userData['accesses'])){

            $formUrl = $url . '/company/' .$get['companyId']. '/user/form';  
            $hasUpdate = true;
        }

        foreach($result as $row) {

            $id = base64_encode($row['id']);

            $action = $hasUpdate ? "<a class='action-button-style btn btn-primary' href='$formUrl/u/$id'>Update</a>" : "";

            $values = array(
                $row['username'],
                $row['fullName'],
                $row['branch'],
                $row['type'],
                $row['contact_no'],
                $action
            );

            $output['data'][] = $values;
        }

        unset($result);

        return $output;
    }
    
    public function autocompleteSuggestions($q, array $userData) {

        $in = "";
       
        $stmtParams = array(
            'q' => "%" . $q['query'] . "%"
        );
        $andWhere = '';
   
        if(isset($userData['branchId'])){
            $andWhere .= ' AND u.`branch_id` = :branchId';
            $stmtParams['branchId'] = base64_decode($userData['branchId']);
        }


         
        $sql = "
            SELECT
                u.`id`,
                CONCAT(u.`first_name` , ' ' , u.`last_name`) AS data,
                CONCAT(u.`first_name` , ' ' , u.`last_name`) AS value
            FROM `user` u
            WHERE CONCAT(u.`first_name` , ' ' , u.`last_name`) LIKE :q
            AND u.`type` != 'Super Admin'
            AND (u.`is_deleted` IS NULL OR  u.`is_deleted` = 0)
            ".$andWhere."
            ORDER BY  CONCAT(u.`first_name` , ' ' , u.`last_name`)
            LIMIT 0,20
        ";

        $query = $this->getEntityManager()->getConnection()->prepare($sql);

        foreach($stmtParams as $k => $v){
            $query->bindValue($k, $v);

        }
       
        $res = $query->executeQuery();
        $result = $res->fetchAllAssociative();
        return $result;
    }
}
