<?php

namespace App\Twig;

use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;

class AppExtension extends AbstractExtension
{
    public function getFilters()
    {
        return [
            new TwigFilter('date_range', [$this, 'dateRange']),
            new TwigFilter('time_ago', [$this, 'timeAgo']),
            new TwigFilter('compute_hours', [$this, 'computeHours']),

        ];
    }

    public function dateRange($from, $to)
    {
        $calcFrom = $from;

        $calcTo = $to;
        return $calcTo->diff($calcFrom)->format("%a");
    }

    function timeAgo( $time )
    {
        $time_difference = time() - strtotime($time);

        if( $time_difference < 1 ) { return 'less than 1 second ago'; }
        $condition = array( 12 * 30 * 24 * 60 * 60 =>  'year',
                    30 * 24 * 60 * 60       =>  'month',
                    24 * 60 * 60            =>  'day',
                    60 * 60                 =>  'hour',
                    60                      =>  'minute',
                    1                       =>  'second'
        );

        foreach( $condition as $secs => $str )
        {
            $d = $time_difference / $secs;

            if( $d >= 1 )
            {
                $t = round( $d );
                return 'about ' . $t . ' ' . $str . ( $t > 1 ? 's' : '' ) . ' ago';
            }
        }
    }

    function computeHours($timeIn, $timeOut, $minimumhour){

        if(is_null($timeIn)) return 0;
        
        $t1 = strtotime($timeIn->format('Y-m-d h:i A'));
        $t2 = strtotime(($timeOut ? $timeOut->format('Y-m-d h:i A') : date('Y-m-d h:i A')));
        $diff =  $t2 - $t1;
        return  $t1 ? ceil($diff / (3600 * $minimumhour)) : 0;
    }
}
?>